/*
    Generate a column made up of a central cylinder or box 
    surrounded by a number of smaller inset cylinders.

    Paul Bourke
    Auckland University Department of Architecture
*/
#include <stdio.h>
#include <math.h>

#define PI       3.141592654
#define TWOPI    6.283185307
#define ABS(x)   ((x) < 0 ? -(x) : (x))

typedef struct {
    double x,y,z;
} XYZ;

char *materialname;
char *objectname;

main(argc,argv)
int argc;
char **argv;
{
    char ctype;
    int i,num;
    double theta,radius1,radius2,overlap,height;
    XYZ p,origin;

    origin.x = 0; origin.y = 0; origin.z = 0;

    if (argc < 9) {
	fprintf(stderr,"Usage: %s ",argv[0]);
	fprintf(stderr,"mat name height (s,c) r1 r2 overlap num ");
	fprintf(stderr,"\n");
	exit(1);
    }
 
    /* Extract the arguments */
    materialname  = argv[1];
    objectname    = argv[2];
    height = atof(argv[3]);
    ctype = argv[4][0];
    if (ctype == 'c' || ctype == 'C')
	ctype = 'c';
    else
	ctype = 's';
    radius1 = atof(argv[5]);
    radius2 = atof(argv[6]);
    overlap = atof(argv[7]);
    if (overlap < -radius2)
	overlap = -radius2;
    num = atoi(argv[8]);

    /* Echo the calling sequence */
    printf("# ");
    for (i=0;i<argc;i++)
	printf("%s ",argv[i]);
    printf("\n");

    /* Generate the central column */
    if (ctype == 's')
	WriteColumn(origin,height,radius1);
    else {
	printf("!genbox %s %s ",materialname,objectname);
	printf("%g %g %g ",2*radius1,2*radius1,height);
	printf("| xform -t %g %g %g\n",-radius1,-radius1,0.0);
    }

    /* Generate the exterior columns */
    for (i=0;i<num;i++) {
	theta = i * TWOPI / (double)num;
	p.x = (radius1 + overlap) * cos(theta);
	p.y = (radius1 + overlap) * sin(theta);
	p.z = 0.0;
	WriteColumn(p,height,radius2);
    }

    exit(0);
}

/*
    Write a Radiance cylinder
*/
WriteColumn(position,height,radius)
XYZ position;
double height,radius;
{
    static int num = 0;

    printf("%s cylinder %s.%d\n",materialname,objectname,num++);
    printf("0\n0\n%ld\n",7);
    printf("   %g %g %g\n",position.x,position.y,position.z);
    printf("   %g %g %g\n",position.x,position.y,position.z + height);
    printf("   %g\n",radius);
    printf("\n");
}
