/*
    Generate a linear staircase

    Paul Bourke
    Auckland University Department of Architecture
*/
#include <stdio.h>
#include <math.h>

#define PI       3.141592654
#define TWOPI    6.283185307
#define RTOD     57.29577951
#define ABS(x)   ((x) < 0 ? -(x) : (x))

typedef struct {
    double x,y,z;
} XYZ;

char *materialname;
char *objectname;

main(argc,argv)
int argc;
char **argv;
{
    int i,nsteps,railposts;
    char leftrightboth = 'n';
    double dx,dy,dz,stepthickness;
    double railthickness,railheight;

    if (argc < 8) {
	fprintf(stderr,"Usage: %s ",argv[0]);
	fprintf(stderr,"mat name dx dy dz nsteps stepthickness\n");
	fprintf(stderr,"   [-r (l,r,b) railheight railposts railthickness]\n");
	exit(1);
    }
 
    /* Extract the arguments */
    materialname  = argv[1];
    objectname    = argv[2];
    dx = atof(argv[3]);
    dy = atof(argv[4]);
    dz = atof(argv[5]);
    nsteps        = atoi(argv[6]);
    stepthickness = atof(argv[7]);

    if (argc > 8) {
	if (argv[8][0] == '-' && argv[8][1] == 'r') {
	    if (argc >= 13) {
		if (argv[9][0] == 'l' || argv[9][0] == 'L')
		    leftrightboth = 'l';
                if (argv[9][0] == 'r' || argv[9][0] == 'R')
                    leftrightboth = 'r';
                if (argv[9][0] == 'b' || argv[9][0] == 'B')
                    leftrightboth = 'b';
		railheight    = atof(argv[10]);
		railposts     = atoi(argv[11]);
		railthickness = atof(argv[12]);
	    } else {
		fprintf(stderr,"Usage: %s ",argv[0]);
		fprintf(stderr,"mat name dx dy dz nsteps stepthickness\n");
		fprintf(stderr,"   [-r (l,r,b) railheight railposts railthickness]\n");
		exit(1);
	    }
	}
    }

    /* Echo the calling sequence */
    printf("# ");
    for (i=0;i<argc;i++)
	printf("%s ",argv[i]);
    printf("\n");

    /* Create the steps */
    for (i=0;i<nsteps;i++) {
	printf("!genbox %s %s_step.%ld ",materialname,objectname,i);
	printf("%g %g %g \\\n",dx/nsteps,dy,stepthickness);
	printf("   | xform -t %g %g %g\n",
	    i*dx/nsteps,-dy/2,i*dz/nsteps);	
    }

    /* Create the handrails */
    for (i=0;i<=railposts;i++) {
	if (leftrightboth == 'r' || leftrightboth == 'b') {
	    printf("!genbox %s %s_rpost.%d ",materialname,objectname,i);
	    printf("%g %g %g \\\n",railthickness,railthickness,railheight);
	    printf("   | xform -t %g %g %g\n",
	        i*dx/railposts,-dy/2,i*dz/railposts);
	}
	if (leftrightboth == 'l' || leftrightboth == 'b') {
            printf("!genbox %s %s_lpost.%ld ",materialname,objectname,i);
            printf("%g %g %g \\\n",railthickness,railthickness,railheight);
            printf("   | xform -t %g %g %g\n",
		i*dx/railposts,dy/2,i*dz/railposts);
	}
    }
/*
    if (leftrightboth == 'r' || leftrightboth == 'b') {
	printf("!genbox %s %s_rrail ",materialname,objectname);
        printf("%g %g %g \\\n",railthickness,railthickness,sqrt(dx*dx+dz*dz));
        printf("   | xform -ry %g -t %g %g %g\n",
	    atan(dx/dz)*RTOD,0.0,-dy/2,railheight);
    }
    if (leftrightboth == 'l' || leftrightboth == 'b') {
        printf("!genbox %s %s_lrail ",materialname,objectname);
        printf("%g %g %g \\\n",railthickness,railthickness,sqrt(dx*dx+dz*dz));
        printf("   | xform -ry %g -t %g %g %g\n",
	    atan(dx/dz)*RTOD,0.0,dy/2,railheight);
    }
*/
    if (leftrightboth == 'r' || leftrightboth == 'b') {
        printf("%s cylinder %s_rrail ",materialname,objectname);
        printf("0\n0\n7 %g %g %g %g %g %g %g",
	    0.0,-dy/2,railheight,dx,-dy/2,dz+railheight,railthickness/2);
    }
    if (leftrightboth == 'l' || leftrightboth == 'b') {
        printf("%s cylinder %s_lrail ",materialname,objectname);
        printf("0\n0\n7 %g %g %g %g %g %g %g",
            0.0,dy/2,railheight,dx,dy/2,dz+railheight,railthickness/2);
    }

    exit(0);
}
