/*
    Generate a vault ceiling for Radiance

    Paul Bourke
    Auckland University Department of Architecture
*/
#include <stdio.h>
#include <math.h>

#define PI       3.141592654
#define TWOPI    6.283185307
#define ABS(x)   ((x) < 0 ? -(x) : (x))

typedef struct {
    double x,y,z;
} XYZ;

char *materialname;
char *objectname;

main(argc,argv)
int argc;
char **argv;
{
    int i,j,k,n;
    double dx,dy,dz,extx,exty;
    XYZ p[9];

    if (argc < 9) {
	fprintf(stderr,"Usage: %s ",argv[0]);
	fprintf(stderr,"material name dx dy dz n extx exty ");
	fprintf(stderr,"\n");
	exit(1);
    }
 
    /* Extract the arguments */
    materialname  = argv[1];
    objectname    = argv[2];
    dx = atof(argv[3]);
    dy = atof(argv[4]);
    dz = atof(argv[5]);
    n  = atoi(argv[6]);
    extx = atof(argv[7]);
    exty = atof(argv[8]);

    /* Echo the calling sequence */
    printf("# ");
    for (i=0;i<argc;i++)
	printf("%s ",argv[i]);
    printf("\n");

    /* Generate the central ceiling */
    for (i=(-n);i<n;i++) {
	for (j=(-n);j<n;j++) {
            p[0].x = 0.5 * i / (double)n;
	    p[0].y = 0.5 * j / (double)n;

	    p[1].x = p[0].x + 0.5 / n;
	    p[2].x = p[1].x;
	    p[3].x = p[0].x;
	    p[4].x = (p[0].x + p[1].x) / 2.0;

	    p[1].y = p[0].y;
	    p[2].y = p[1].y + 0.5 / n;
	    p[3].y = p[2].y;
	    p[4].y = (p[0].y + p[3].y) / 2.0;

	    for (k=0;k<5;k++) {
		if ((p[k].y < p[k].x && p[k].x + p[k].y < 0) 
		    || (p[k].y > p[k].x && p[k].x + p[k].y > 0))
		    p[k].z = sqrt(0.25001 - p[k].x*p[k].x);
		else
                    p[k].z = sqrt(0.25001 - p[k].y*p[k].y);
	    }
	    
	    for (k=0;k<5;k++) {
		p[k].x *= dx;
		p[k].y *= dy;
		p[k].z *= dz;
	    }
	    
	    WriteFacet(p[0],p[1],p[4]);
            WriteFacet(p[1],p[2],p[4]);
            WriteFacet(p[2],p[3],p[4]);
            WriteFacet(p[3],p[0],p[4]);

	    if (i == n-1 && extx > 0) {
		p[5] = p[1];
		p[6] = p[2];
		p[5].x += extx;
		p[6].x += extx;
		WriteFacet(p[1],p[5],p[6]);
		WriteFacet(p[6],p[2],p[1]);
	    }
            if (i == -n && extx > 0) {
		p[7] = p[0];
		p[8] = p[3];
                p[7].x -= extx;
                p[8].x -= extx;
                WriteFacet(p[0],p[3],p[8]);
                WriteFacet(p[8],p[7],p[0]);
            }
	    if (j == -n && exty > 0) {
		p[5] = p[0];
		p[6] = p[1];
                p[5].y -= exty;
                p[6].y -= exty;
                WriteFacet(p[0],p[5],p[6]);
                WriteFacet(p[6],p[1],p[0]);
	    }
            if (j == n-1 && exty > 0) {
		p[7] = p[3];
		p[8] = p[2];
                p[7].y += exty;
                p[8].y += exty;
                WriteFacet(p[2],p[8],p[7]);
                WriteFacet(p[7],p[3],p[2]);
            }
	}
    }

    exit(0);
}

/*
    Write a facet in Radiance format, as a polygon
*/
WriteFacet(p1,p2,p3)
XYZ p1,p2,p3;
{
    int i;
    static int num;

    printf("%s polygon %s.%d\n",materialname,objectname,num++);
    printf("0\n0\n%ld\n",9);
    printf("   %g %g %g\n",p1.x,p1.y,p1.z);
    printf("   %g %g %g\n",p3.x,p3.y,p3.z);
    printf("   %g %g %g\n",p2.x,p2.y,p2.z);
    printf("\n");
}
